#include <WiFi.h>
#include <ESPAsyncWebServer.h>
#include <SPIFFS.h>
#include <WebSocketsServer.h>
#include <DHT.h>

const char *ssid = "UNTEL MALLQUI";
const char *password = "c7hd5e9j";

const uint8_t dhtPin = 18;
DHT dht(dhtPin, DHT22);
float temperature = 0.0;
float humidity = 0.0;

AsyncWebServer server(80);
WebSocketsServer WebSocket(81);
const unsigned long sendInterval = 500;

void setup() {
  Serial.begin(115200);
  WiFi.begin(ssid, password);
  while (WiFi.status() != WL_CONNECTED) { delay(500); }

  Serial.println("[WIFI] Connected: ");
  Serial.println(WiFi.localIP());

  dht.begin();

  if (SPIFFS.begin(true)) {
    Serial.println("SPIFFS montada correctamente");

    File root = SPIFFS.open("/");
    File file = root.openNextFile();
    while (file) {
      Serial.print("Archivo: ");
      Serial.println(file.name());
      file = root.openNextFile();
    }
  } else {
    Serial.println("Error en el montado");
  }

  server.on("/", HTTP_GET, [](AsyncWebServerRequest *request) {
    request->send(SPIFFS, "/index_2.html", "text/html");
  });

  server.onNotFound(notFound);
  server.begin();
  WebSocket.begin();
  WebSocket.onEvent([](uint8_t num, WStype_t type, uint8_t *payload, size_t length) {
    handleWebSocketMessage(num, type, payload, length);
  });
}

void loop() {
  WebSocket.loop();

  static uint32_t prevMillis = 0;
  if (millis() - prevMillis >= sendInterval) {
    prevMillis = millis();

    temperature = dht.readTemperature();
    humidity = dht.readHumidity();

    if (isnan(humidity) || isnan(temperature))
      return;
    sendData();
  }
}

void sendData() {
  String data = "{\"temperature\":" + String(temperature) + ",\"humidity\":" + String(humidity) + "}";

  Serial.println(data);
  WebSocket.broadcastTXT(data);
}

void handleWebSocketMessage(uint8_t num, WStype_t type, uint8_t *payload, size_t length) {
  char localBuffer[256]; // Local buffer declared inside the function

  switch (type) {
    case WStype_DISCONNECTED:
      Serial.printf("[%u] Desconectado!\n", num);
      break;
    case WStype_CONNECTED: {
      IPAddress ip = WebSocket.remoteIP(num);
      Serial.printf("[%u] Conectado en %d.%d.%d.%d url: %s\n", num, ip[0], ip[1], ip[2], ip[3]);
      break;
    }
    case WStype_TEXT:
      if (length < sizeof(localBuffer)) {
        strncpy(localBuffer, (char*)payload, length);
        localBuffer[length] = '\0'; // Null-terminate the string
        Serial.printf("[%u] Texto recibido: %s\n", num, localBuffer);
        // Handle the received text message stored in localBuffer
      } else {
        Serial.printf("[%u] Mensaje demasiado largo para el buffer\n", num);
      }
      break;
    // Handle other cases if necessary
    default:
      break;
  }
}

void notFound(AsyncWebServerRequest *request) {
  request->send(404, "text/plain", "Pagina no encontrada");
}
